<?php

/* 
* Author:    Levi D.
* cc_tools:  180116-000003
* Purpose:   Cron meant to run daily to record data on the usage of items within our site.
* Created:   2018-01-26
*/
if (!defined('DOCROOT')) 
{
	$docroot = get_cfg_var('doc_root');
	define('DOCROOT', $docroot);
}

if(isset($_POST['username']))
	$username = $_POST['username'];
else if(isset($_GET['username']))
	$username = $_GET['username'];

if(isset($_POST['password']))
	$password = $_POST['password'];
else if(isset($_GET['password']))
	$password = $_GET['password'];

/************* Agent Authentication ***************/

// Set up and call the AgentAuthenticator
require_once (DOCROOT . '/include/services/AgentAuthenticator.phph');

// On failure, this includes the Access Denied page and then exits,
// preventing the rest of the page from running.
$account = AgentAuthenticator::authenticateCookieOrCredentials($username, $password);

/************* End agent authentication ***********/

// set a constant
define('NAME', "item_usage");

// Set up versioned namespace for the Connect PHP API
use RightNow\Connect\v1_4 as RNCPHP;
$context = RNCPHP\ConnectAPI::getCurrentContext();
$context->ApplicationContext = NAME;

//----------- Ready to proceed with script --------//

// constants

// general use
define('BROWSER_DEBUG', false);  // used to turn on/off browser output for debugging purposes
define('REPORT_LIMIT', 10000);  // NOTE this is the same max limit imposed on the report run in CPHP, but defining so we can control if we want

// end constants


try
{
	BrowserDebug("START");
	
	// get all the item usage meta data
	$result = RNCPHP\ROQL::queryObject("USE OPERATIONAL; SELECT CC.ItemUsageMeta FROM CC.ItemUsageMeta ORDER BY CC.ItemUsageMeta.ID")->next();
	//BrowserDebug($result);
	
	// iterate through the results
	while($result_meta = $result->next())
	{
		/*
		$result_meta->ID;
		$result_meta->LookupName;
		$result_meta->DisplayOrder;
		$result_meta->Name;
		$result_meta->RPTCount;
		$result_meta->RPTItemID;
		$result_meta->RPTItemName;
		$result_meta->RPTLastUsed;
		$result_meta->ReportID;
		$result_meta->RPTTimeFilterName;
		$result_meta->FilterByItemName;
		$result_meta->Disabled;
		BrowserDebug($result_meta);
		*/

		BrowserDebug("start processing {$result_meta->Name} records");
		if($result_meta->FilterByItemName)
			BrowserDebug("doing lookup by ItemName");
		else
			BrowserDebug("doing lookup by ItemID");
		
		ProcessMeta($result_meta);
		BrowserDebug("end processing {$result_meta->Name} records");
	}
	
	BrowserDebug("END");
	
} // try

catch(\Exception $e)
{
	ESDLog('exception - in main body', $e, 3);
	BrowserDebug($e);
}
catch(RNCPHP\ConnectAPIErrorBase $e)
{
	ESDLog('exception - in main body', $e, 3);
	BrowserDebug($e);
}


/**
 * processes a provided CC\ItemUsageMeta object
 * @param $meta - object, an instance of a CC\ItemUsageMeta object
 */
function ProcessMeta($meta)
{
	try
	{
		// do nothing if disabled
		if($meta->Disabled)
		{
			BrowserDebug("processing for {$meta->Name} records is Disabled");
			return;
		}
		
		// get the max LastUsed for this Meta on CC$ItemUsage
		$last_used = LastUsed($meta->ID);
		//BrowserDebug($last_used);
		
		// return if last_used is -1... that means something unexpected happened
		if($last_used == -1)
		{
			BrowserDebug("last_used = -1, something bad happened");
			return;
		}
		else
		{
			// we should have a empty/blank/null  value or a date in ISO 8601 format that we will use to set the filter and run the report
			//BrowserDebug("last_used is empty/blank/null or in ISO 8601 format date");
			
			// get our report
			$report = RNCPHP\AnalyticsReport::fetch($meta->ReportID);
			
			$filter = new RNCPHP\AnalyticsReportSearchFilter;
			$filter->Name = $meta->RPTTimeFilterName;
			
			// only set a value if we have it
			if(!empty($last_used))
				$filter->Values[] = $last_used;
			
			$filter->Operator = new RNCPHP\NamedIDOptList;
			$filter->Operator->ID = 5; // >, i.e greater than
			
			$filters = new RNCPHP\AnalyticsReportSearchFilterArray;
			$filters[] = $filter;
			
			$report_start = 0;
			$report_count = REPORT_LIMIT;
			$report_run = 0;
			$update_count = 0;
			$new_count = 0;
			// this is to handle if we return the same number of records as the limit... that means we have more data and need to run again
			while($report_count == REPORT_LIMIT)
			{
				$report_results = $report->run($report_start, $filters, REPORT_LIMIT);
				$report_count = $report_results->count();
				//BrowserDebug($report_count);
				//BrowserDebug($report_results);
				while($report_row = $report_results->next())
				{
					//BrowserDebug($report_row);
					
					// set our first statement
					if($meta->FilterByItemName)
						$first = "ItemName = '" . RNCPHP\ROQL::escapeString($report_row[$meta->RPTItemName]) . "' AND Meta = " . $meta->ID;
					else
						$first = "ItemID = " . $report_row[$meta->RPTItemID] . " AND Meta = " . $meta->ID;
					
					//BrowserDebug($first);
					$record = RNCPHP\CC\ItemUsage::first($first);
					//BrowserDebug($record);

					// update existing
					if($record)
					{
						$record->Count += $report_row[$meta->RPTCount];
						$record->LastUsed = strtotime(trim($report_row[$meta->RPTLastUsed], "'"));
						$record->save(RNCPHP\RNObject::SuppressAll);
						$update_count++;
					}
					// create new
					else
					{
						$new_record = new RNCPHP\CC\ItemUsage;
						if($meta->FilterByItemName !== true)
							$new_record->ItemID = $report_row[$meta->RPTItemID];
						
						if($report_row[$meta->RPTItemName] != "" || $meta->FilterByItemName == true)
							$new_record->ItemName = $report_row[$meta->RPTItemName];
						
						$new_record->Count = $report_row[$meta->RPTCount];
						$new_record->LastUsed = strtotime(trim($report_row[$meta->RPTLastUsed], "'"));
						$new_record->Meta = $meta;
						$new_record->save(RNCPHP\RNObject::SuppressAll);
						$new_count++;
					}

				} // while($report_row = $report_results->next())
				
				// increase by our report/record limit... this for if we need to run the report again but at a different offset
				$report_start += REPORT_LIMIT;
				$report_run++;
				
				BrowserDebug("report run {$report_run} returned {$report_count} rows");
				
			} // while($report_count == REPORT_LIMIT)
				
			$total_count = $new_count + $update_count;
			BrowserDebug("{$total_count} total records processed");
			BrowserDebug("{$new_count} records created");
			BrowserDebug("{$update_count} records updated");
			
			if($total_count > 0)
			{
				BrowserDebug("pre Commit");
				RNCPHP\ConnectAPI::commit();
				BrowserDebug("post Commit");
			}
			
		} // else
		
	} // try
	
	catch(\Exception $e)
	{
		ESDLog('exception - in ProcessMeta function', $e, 3);
		BrowserDebug($e);
		BrowserDebug("pre rollback");
		RNCPHP\ConnectAPI::rollback();
		BrowserDebug("post rollback");
	}
	catch(RNCPHP\ConnectAPIErrorBase $e)
	{
		ESDLog('exception - in ProcessMeta function', $e, 3);
		BrowserDebug($e);
		BrowserDebug("pre rollback");
		RNCPHP\ConnectAPI::rollback();
		BrowserDebug("post rollback");
	}
	
} // function ProcessMeta


/**
 * provides the most recent LastUsed value of given type of record in CC$ItemUsage
 * @param $meta_id - int, the ID value of a CC$ItemUsageType (this corresponds to CC$ItemUsage.Meta)
 * @returns - -1 if there was an issue OR null/blank/empty if there is no max value OR returns a date in ISO 8601 format (e.g. 2018-03-15T17:27:00.000Z)
 */
function LastUsed($meta_id)
{
	try
	{
		$last = -1;
		
		$query = "USE OPERATIONAL; SELECT max(iu.LastUsed) AS 'LastUsed' FROM CC.ItemUsage iu WHERE iu.Meta.ID = " . $meta_id;
		//BrowserDebug($query);

		// there should only ever be 1 record returned
		$record = RNCPHP\ROQL::query($query)->next()->next();
		//BrowserDebug($record);
		
		$last = $record['LastUsed'];
		//BrowserDebug($last);
		
	} // try
	
	catch(\Exception $e)
	{
		ESDLog('exception - in LastUsed function', $e, 3);
		BrowserDebug($e);
	}
	catch(RNCPHP\ConnectAPIErrorBase $e)
	{
		ESDLog('exception - in LastUsed function', $e, 3);
		BrowserDebug($e);
	}
	
	return $last;
	
} // function LastUsed


/**
 * outputs a specified debug message in a browser; only outputs if BROWSER_DEBUG constant is true
 * @param $message - string/object/array, the information to output
 */
function BrowserDebug($message)
{
	try
	{
		if(BROWSER_DEBUG)
			echo "<pre>" . print_r($message, true) . "</pre>";
		
	}
	catch(\Exception $e)
	{
		echo print_r($e, true);
	}
	catch(RNCPHP\ConnectAPIErrorBase $e)
	{
		echo print_r($e, true);
	}
	
	return;
	
} // function BrowserDebug


/**
 * creates a record in ESD$IntegrationLog, typically used for recording errors and exceptions but can also be used to log for debugging
 * @param $message - string, the summary of the log (max size of 255)
 * @param $stack - string/object/arry, the details of the issues (e.g. put exception details here)... it will be put in print_r($stack, true)
 * @param $type - int, 1 = info, 2 = warn, 3 = error
 */
function ESDlog($message, $stack, $type)
{
	try
	{
		$log = new RNCPHP\ESD\IntegrationLog();
		$log->AccountID = 2633; //Levi Dyckman
		$log->Message = $message;
		$log->Stack = print_r($stack, true);
		$log->Type = $type;
		$log->Category="Cron";
		$log->Name = NAME;
		$log->save(RNCPHP\RNObject::SuppressAll);
	}
	catch(\Exception $e)
	{
		echo print_r($e, true);
	}
	catch(RNCPHP\ConnectAPIErrorBase $e)
	{
		echo print_r($e, true);
	}
	
	return;
	
} //function ESDlog

?>